@@ -347,6 +347,9 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
     char   *dsn_orcpt = 0;
     int     n;
     int     have_log_client_attr = 0;
+    static const char env_rec_types[] = REC_TYPE_ENVELOPE REC_TYPE_EXTRACT;
+    static const char extra_rec_type[] = {REC_TYPE_XTRA, 0};
+    const char *expected_rec_types;
 
     /*
      * Initialize. No early returns or we have a memory leak.
@@ -411,12 +414,14 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
      * mailing lists.
      */
     for (;;) {
+	expected_rec_types = env_rec_types;
 	if ((curr_offset = vstream_ftell(message->fp)) < 0)
 	    msg_fatal("vstream_ftell %s: %m", VSTREAM_PATH(message->fp));
 	if (curr_offset == message->data_offset && curr_offset > 0) {
 	    if (vstream_fseek(message->fp, message->data_size, SEEK_CUR) < 0)
 		msg_fatal("seek file %s: %m", VSTREAM_PATH(message->fp));
 	    curr_offset += message->data_size;
+	    expected_rec_types = extra_rec_type;
 	}
 	rec_type = rec_get_raw(message->fp, buf, 0, REC_FLAG_NONE);
 	start = vstring_str(buf);
@@ -433,6 +438,12 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 		     message->queue_id);
 	    break;
 	}
+	if (strchr(expected_rec_types, rec_type) == 0) {
+	    msg_warn("Unexpected record type '%c' at offset %ld",
+		     rec_type, (long) curr_offset);
+	    rec_type = REC_TYPE_ERROR;
+	    break;
+	}
 	if (rec_type == REC_TYPE_END) {
 	    message->rflags |= QMGR_READ_FLAG_SEEN_ALL_NON_RCPT;
 	    break;
@@ -446,7 +457,7 @@ static int qmgr_message_read(QMGR_MESSAGE *message)
 	 */
 	if (rec_type == REC_TYPE_ATTR) {
 	    if ((error_text = split_nameval(start, &name, &value)) != 0) {
-		msg_warn("%s: ignoring bad attribute: %s: %.200s",
+		msg_warn("%s: bad attribute record: %s: %.200s",
 			 message->queue_id, error_text, start);
 		rec_type = REC_TYPE_ERROR;
 		break;
@@ -1137,7 +1148,7 @@ static void qmgr_message_resolve(QMGR_MESSAGE *message)
 	}
 
 	/*
-	 * Redirect a force-expired message without defer log to the retry
+	 * Redirect a forced-to-expire message without defer log to the retry
 	 * service, so that its defer log will contain an appropriate reason.
 	 * Do not redirect such a message to the error service, because if
 	 * that request fails, a defer log would be created with reason
@@ -1525,14 +1536,14 @@ QMGR_MESSAGE *qmgr_message_alloc(const char *queue_name, const char *queue_id,
 	    msg_fatal("fchmod %s: %m", VSTREAM_PATH(message->fp));
 
 	/*
-	 * If this message is force-expired, use the existing defer logfile
-	 * records and do not assign any deliveries, leaving the refcount at
-	 * zero. If this message is force-expired, but no defer logfile
-	 * records are available, assign deliveries to the retry transport so
-	 * that the sender will still find out what recipients are affected
-	 * and why. Either way, do not assign normal deliveries because that
-	 * would be undesirable especially with mail that was expired in the
-	 * 'hold' queue.
+	 * If this message is forced to expire, use the existing defer
+	 * logfile records and do not assign any deliveries, leaving the
+	 * refcount at zero. If this message is forced to expire, but no
+	 * defer logfile records are available, assign deliveries to the
+	 * retry transport so that the sender will still find out what
+	 * recipients are affected and why. Either way, do not assign normal
+	 * deliveries because that would be undesirable especially with mail
+	 * that was expired in the 'hold' queue.
 	 */
 	if ((message->qflags & QMGR_FORCE_EXPIRE) != 0
 	    && stat(mail_queue_path((VSTRING *) 0, MAIL_QUEUE_DEFER,
