@@ -427,11 +427,19 @@ static DICT_OPEN_EXTEND_FN dict_open_extend_hook;
 static DICT_MAPNAMES_EXTEND_FN dict_mapnames_extend_hook;
 
  /*
-  * Workaround.
+  * Workaround: define global variables here to control database cache sizes.
+  * When a database driver is dynamically loaded, global control variables
+  * cannot simply be owned by the loadable objects because that would result
+  * in build-time linker errors.
   */
 DEFINE_DICT_LMDB_MAP_SIZE;
 DEFINE_DICT_DB_CACHE_SIZE;
 
+ /*
+  * Replace obscure code with a more readable expression.
+  */
+#define NEED_DICT_OPEN_INIT()	(dict_open_hash == 0)
+
 /* dict_open_init - one-off initialization */
 
 static void dict_open_init(void)
@@ -439,7 +447,7 @@ static void dict_open_init(void)
     const char *myname = "dict_open_init";
     const DICT_OPEN_INFO *dp;
 
-    if (dict_open_hash != 0)
+    if (!NEED_DICT_OPEN_INIT())
 	msg_panic("%s: multiple initialization", myname);
     dict_open_hash = htable_create(10);
 
@@ -476,7 +484,7 @@ DICT   *dict_open3(const char *dict_type, const char *dict_name,
     if (*dict_type == 0 || *dict_name == 0)
 	msg_fatal("open dictionary: expecting \"type:name\" form instead of \"%s:%s\"",
 		  dict_type, dict_name);
-    if (dict_open_hash == 0)
+    if (NEED_DICT_OPEN_INIT())
 	dict_open_init();
     if ((dp = dict_open_lookup(dict_type)) == 0)
 	return (dict_surrogate(dict_type, dict_name, open_flags, dict_flags,
@@ -516,7 +524,7 @@ void    dict_open_register(const DICT_OPEN_INFO *dp)
 
     if (msg_verbose > 1)
 	msg_info("%s: %s", myname, dp->type);
-    if (dict_open_hash == 0)
+    if (NEED_DICT_OPEN_INIT())
 	dict_open_init();
     if (htable_find(dict_open_hash, dp->type))
 	msg_panic("%s: dictionary type exists: %s", myname, dp->type);
@@ -550,13 +558,6 @@ DICT_OPEN_EXTEND_FN dict_open_extend(DICT_OPEN_EXTEND_FN new_cb)
     return (old_cb);
 }
 
-/* dict_sort_alpha_cpp - qsort() callback */
-
-static int dict_sort_alpha_cpp(const void *a, const void *b)
-{
-    return (strcmp(((char **) a)[0], ((char **) b)[0]));
-}
-
 /* dict_mapnames - return an ARGV of available map_names */
 
 ARGV   *dict_mapnames()
@@ -566,7 +567,7 @@ ARGV   *dict_mapnames()
     DICT_OPEN_INFO *dp;
     ARGV   *mapnames;
 
-    if (dict_open_hash == 0)
+    if (NEED_DICT_OPEN_INIT())
 	dict_open_init();
     mapnames = argv_alloc(dict_open_hash->used + 1);
     for (ht_info = ht = htable_list(dict_open_hash); *ht; ht++) {
@@ -575,8 +576,9 @@ ARGV   *dict_mapnames()
     }
     if (dict_mapnames_extend_hook != 0)
 	(void) dict_mapnames_extend_hook(mapnames);
-    qsort((void *) mapnames->argv, mapnames->argc, sizeof(mapnames->argv[0]),
-	  dict_sort_alpha_cpp);
+    argv_qsort(mapnames, (ARGV_COMPAR_FN) 0);
+    /* In case some drivers have been loaded dynamically. */
+    argv_uniq(mapnames, (ARGV_COMPAR_FN) 0);
     myfree((void *) ht_info);
     argv_terminate(mapnames);
     return mapnames;
@@ -608,7 +610,7 @@ void    dict_type_override(DICT *dict, const char *type)
   */
 int     main(int argc, char **argv)
 {
-    dict_test(argc, argv);
+    dict_cli(argc, argv);
     return (0);
 }
 
